# This script initializes the LaTeX author resources provided for
# Frontiers journals.
# https://www.frontiersin.org/journals/applied-mathematics-and-statistics/sections/optimization#author-guidelines

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.frontiersin.org/design/zip/Frontiers_LaTeX_Templates.zip')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Journal of Optimization Theory and Applications (JOTA).
# https://www.springer.com/journal/10957/submission-guidelines

import re
import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://media.springer.com/full/springer-instructions-for-authors-assets/zip/1705114_LaTeX%20JOTA.zip')

        # Fix the missing comment in the spmpsci.bst file.
        data = open('spmpsci.bst', 'r', newline = '\r\n').read()
        data = re.sub(r'`vonx', r'%% `vonx', data)
        open('spmpsci.bst', 'w').write(data)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# American Chemical Society journals.
# https://pubs.acs.org/page/4authors/submission/tex.html

import shutil
import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from CTAN.
        utilities.get_archive('https://mirrors.ctan.org/macros/latex/contrib/achemso.zip', junk = 1)

        # Run the installer, which is going to create the .cls, .bst, .cfg, .sty files.
        run(['latex', 'achemso.ins'], stderr = STDOUT)

        # Correct misnamed files.
        shutil.move('achemso-inoraj.cfg', 'achemso-inocaj.cfg')
        shutil.move('achemso-appccd.cfg', 'achemso-apaccd.cfg')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Numerical Linear Algebra with Applications (NLA).
# https://onlinelibrary.wiley.com/page/journal/10991506/homepage/forauthors.html
# https://onlinelibrary.wiley.com/page/journal/10991506/homepage/la_tex_class_file.htm

import glob
import os
import re
import shutil
import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher, using wget.
        utilities.get_archive('https://onlinelibrary.wiley.com/pb-assets/assets/10991506/VANCOUVER-stix_New-1658936365330.zip', junk = 1, method = "wget")

        # Move the content of the VANCOUVER/ directory one up.
        for f in glob.glob(f'VANCOUVER/*'):
            # Proceed only if the object is a file (not a directory).
            if os.path.isfile(f):
                destination = f
                destination = re.sub(r'.*?/', '', destination, count = 1)

                # Construct the full path of the destination.
                # This may be necessary in order to allow files to be overwritten.
                destination = os.path.join(os.getcwd(), destination)
                shutil.move(f, destination)

        # The WileyNJD-v2.cls file requires WileyNJD-VANCOUVER.bst but the author resources contain wileyNJD-VANCOUVER.bst.
        # We rename the file since symbolic links to files differing only w.r.t. the case cause issues on macOS.
        source = 'wileyNJD-VANCOUVER.bst'
        destination = os.path.join(os.getcwd(), 'WileyNJD-VANCOUVER.bst')
        shutil.move(source, destination)

        # Wiley journals use different versions of WileyNJD-v2.cls under the same name.
        # We rename the file in a journal specific way for the purpose of disambiguation.
        utilities.link_or_copy('WileyNJD-v2.cls', 'WileyNJD-v2-nla.cls')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# IFAC-PapersOnLine (IFAC).
# https://www.ifac-control.org/conferences/authors-guide

import os
import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.ifac-control.org/conferences/resolveuid/b0e4734736878819db179d76290a8f78', junk = 1)

        # Remove some files.
        os.remove('.gitignore')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Electronic Transactions on Numerical Analysis (ETNA).
# https://etna.ricam.oeaw.ac.at/submissions/latex/
# https://etna.ricam.oeaw.ac.at/submissions/

import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get the LaTeX author resources from the publisher.
        utilities.get_file('https://etna.ricam.oeaw.ac.at/submissions/latex/example.tex')
        utilities.get_file('https://etna.ricam.oeaw.ac.at/submissions/latex/etna.cls')
        utilities.get_file('https://etna.ricam.oeaw.ac.at/submissions/latex/etna.bst')
        utilities.get_file('https://etna.ricam.oeaw.ac.at/submissions/ETNA_guidelines.pdf')

        # Compile the template file.
        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'example.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'example.tex'], stderr = STDOUT)

        # The file etna.cls has some issues.
        # One issue is that it redefines \@begintheorem in a way incompatible with amsthm.sty.
        # However we would like to use amsthm.sty, which offers cleveref.sty support with a combined counter for all
        # theorem-like environments.
        # Also, it defines remark and example environments which cause some trouble.
        # We apply a patch created via
        #   diff -u --ignore-trailing-space etna.cls etna-scoop.cls > etna.cls.patch
        utilities.apply_patch('etna.cls.patch')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



import cloudscraper
import datetime
import glob
import importlib.metadata
import os
import patch
import re
import requests
import shutil
import sys
import tempfile
import setuptools_git_versioning
from subprocess import run, STDOUT


# Define a neutral user agent.
userAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64)"

def get_file(url, verify = True, method = "requests"):
    """
    Retrieves a file from a given url, writes it to the current directory and
    returns the file name.
    """

    if method == "wget":
        # The 'wget' option uses the system-level 'wget' which is capable of
        # overcoming some protections against automated downloads.
        print('Retrieving {0:s} using wget'.format(url))
        filename = url.split("/")[-1]
        filename = filename.split("?")[0]
        print('File name from URL: {0:s}'.format(filename))
        try:
            # We need 'shell = True' likely since some download protection mechanisms
            # check the environment. '--timestamping' is used in order to make wget
            # overwrite possibly existing copies of the file.
            command = 'wget --user-agent="' + userAgent + '" --timestamping --trust-server-names ' + url
            print('Running {0:s}'.format(command))
            run(command, shell = True, stderr = STDOUT)
        except FileNotFoundError:
            print('wget does not seem to be available on your system. Unable to retrieve {0:s}. The template will not be available.'.format(url))

    elif method == "requests":
        # Retrieve the URL using 'requests'.
        print('Retrieving {0:s} using requests'.format(url))
        r = requests.get(url, verify = verify)

        # If the request was rejected, try again using a neutral user agent.
        if not r.ok:
            headers = {"User-Agent": userAgent}
            r = requests.get(url, verify = verify, headers = headers)

        # Retrieve the file name.
        filename = get_filename(r)

        # Write out the raw content.
        with open(filename, 'wb') as f:
            f.write(r.content)

    elif method == "cloudscraper":
        # Retrieve the URL using 'cloudscraper'.
        print('Retrieving {0:s} using cloudscraper'.format(url))
        scraper = cloudscraper.create_scraper()
        r = scraper.get(url, verify = verify)

        # Retrieve the file name.
        filename = get_filename(r)

        # Write out the raw content.
        with open(filename, 'wb') as f:
            f.write(r.content)

    else:
        # Unknown download method.
        print(f'Unknown download method {method}.')
        sys.exit(1)

    # Return the file name.
    return filename


def get_filename(r):
    """
    Retrieves the file name from the request r.
    """
    # Construct the file name either as specified by the server, or else from
    # the URL. The header field 'Content-Disposition', if it exists, typically
    # reads
    #   Content-Disposition: attachment
    #   Content-Disposition: attachment; filename="file.zip"
    #   Content-Disposition: inline; filename="file.zip"

    # Try the 'Content-Disposition' header field.
    filename = r.headers.get("Content-Disposition")
    if filename and re.search('^(attachment|inline)', filename):
        filename = re.findall(r'^(?:attachment|inline); filename=(.*)$', filename)[0]
        # Strip enclosing single or double quotes from the file name.
        filename = filename.strip('\'')
        filename = filename.strip('\"')

    # If this is not successful, extract the file name from the URL.
    if not filename:
        filename = r.url.split("/")[-1]
        filename = filename.split("?")[0]
        print('File name from URL: {0:s}'.format(filename))
    else:
        print('File name from header: {0:s}'.format(filename))

    return filename


def get_archive(url, verify = True, junk = 0, method = "requests"):
    """
    Retrieves an archive file from a given url, extracts it and writes the
    content to the current directory with the required number of leading
    directories junked from the path.
    """

    # Retrieve the archive file.
    filename = get_file(url, verify = verify, method = method)

    # Unpack the archive into a temporary directory (plus whatever relative paths
    # there may be associated with the archive's content).
    tmpdir = 'tmp'
    print('Unpacking {0:s} into {1:s}'.format(filename, tmpdir))
    shutil.unpack_archive(filename, tmpdir)

    # Move the archive's content into the current directory, i.e., junk the
    # tmpdir and whichever number of additional path components as indicated
    # through the junk parameter.
    for f in glob.glob(f'{tmpdir}/**/*', recursive = True) + glob.glob(f'{tmpdir}/**/.*', recursive = True):

        # Proceed only if the object is a file (not a directory).
        if os.path.isfile(f):
            # Construct the relative path of the destination.
            # Remove the tmpdir/ component from the left.
            regex = r'^' + tmpdir + r'/'
            destination = f
            destination = re.sub(regex, '', destination, count = 1)

            # Remove up to 'junk' many additional path components from the left.
            if junk > 0:
                destination = re.sub(r'.*?/', '', destination, count = junk)

            # Construct the full path of the destination.
            # This may be necessary in order to allow files to be overwritten.
            destination = os.path.join(os.getcwd(), destination)

            # Make sure the destination directory exists.
            if not os.path.exists(os.path.dirname(destination)):
                os.mkdir(os.path.dirname(destination))

            # Move the file.
            print('Moving {0:s} to {1:s}'.format(f, destination))
            shutil.move(f, destination)

    # Remove the archive folder
    shutil.rmtree(tmpdir)


def apply_patch(patchFile):
    """
    Applies the patches described in the patch file.
    """
    # Since https://pypi.org/project/patch/ modifies the source file in place and
    # does not honor the target file, we create a work around. For each patch
    # item, we copy its source file into a temporary file, then apply the patch
    # file, move the patched source files to their destination and move the
    # temporary copies back to the source files.

    # Read the patch data.
    patchData = patch.fromfile(patchFile)

    # Create lists of source, target, and temporary file names.
    sourceFiles = []
    tempFiles = []
    targetFiles = []

    # Iterate over the files to be patched.
    for patchItem in patchData.items:

        # Get and store the files affected by the current patch item.
        sourceFile = patchItem.source
        targetFile = patchItem.target
        tempFile = tempfile.mkstemp()[1]

        # Copy the source file to the temporary file.
        shutil.copy(sourceFile, tempFile)

        # Store the file names.
        sourceFiles.append(sourceFile)
        targetFiles.append(targetFile)
        tempFiles.append(tempFile)

    # Apply the patch set to all files.
    patchData.apply()

    # Iterate over the files again.
    for sourceFile, targetFile, tempFile in zip(sourceFiles, targetFiles, tempFiles):
        # Move the patched source file to its target.
        shutil.move(sourceFile, targetFile)

        # Retrieve the original source file from the temporary file.
        shutil.move(tempFile, sourceFile)


def link_or_copy(source, destination):
    """
    Tries to make destination a symbolic link pointing to source.
    If this fails, copies destination to source instead.
    """
    # Remove the destination if it exists.
    if os.path.isfile(destination):
        os.remove(destination)

    # Try to create a symbolic link.
    # If this fails, copy the source to the destination instead.
    try:
        os.symlink(source, destination)
    except:
        shutil.copy(source, destination)


def mac2unix(source):
    """
    Converts macOS-style \r line endings to unix-style \n line endings
    to achieve compatibility with diff.
    """
    #  https://stackoverflow.com/questions/75202752/replace-mcontrol-m-character-in-a-text-file-in-python
    with open(source, "rb") as input_file:
        contents = input_file.read().replace(b"\r",b"\n")
    with open(source, "wb") as output_file:
        output_file.write(contents)

def protect(source):
    """
    Renames the source file to a reproducible but unlikely name to protect it from being
    overwritten by content of the same name in the publisher's archive.
    """
    destination = 'scoop-tmp-' + source
    os.replace(source, destination)


def restore(source):
    """
    Renames the source file to a reproducible alternative name with prefix
    'scoop-disambiguation-' and restores the original file.
    """
    original = 'scoop-tmp-' + source
    alternative = 'scoop-disambiguation-' + source
    os.replace(source, alternative)
    os.replace(original, source)


def remove_time_version_stamp():
    """
    Remove an existing time and version stamp.
    """
    filename = 'SCOOP-STAMP'
    if os.path.isfile(filename):
        os.remove(filename)


def write_time_version_stamp():
    """
    Write a time and version stamp to indicate successful completion.
    """
    filename = 'SCOOP-STAMP'
    try:
        # Get the version number from the package data.
        version = importlib.metadata.version("scoop-template-engine")
    except:
        # Get the version number from the most recent git tag,
        # defaulting to "0.0.0".
        version = str(setuptools_git_versioning.get_tag() or "0.0.0").lstrip("v")

    # Generate the date string.
    dateTime = datetime.datetime.utcnow().strftime("%Y%m%d-%H:%M:%S UTC")

    # Write the time and version stamp.
    stampString = "{version:s}\n{dateTime:s}\n".format(version = version, dateTime = dateTime)
    with open(filename, 'w') as f:
        f.write(stampString)



# This script initializes the LaTeX author resources provided for
# International Journal of Computational Geometry and Applications (IJCGA).
# https://www.worldscientific.com/page/authors/journal-stylefiles

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.worldscientific.com/sda/1037/ijcga-2e.zip')

        # Get the separate style manual as well.
        utilities.get_file('https://www.worldscientific.com/sda/1037/ws-ijcga.pdf')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Journal für die reine und angewandte Mathematik (Crelle).
# https://www.degruyter.com/journal/key/crll/html

import re
import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.degruyter.com/publication/journal_key/CRLL/downloadAsset/CRLL_LaTeX_template.zip')

        # Activate the template file by uncommenting one of the \documentclass lines.
        data = open('article.tex', 'r').read()
        data = re.sub(r'%% (\\documentclass\[SecEq)', r'\1', data)
        open('article.tex', 'w').write(data)

        # Compile the template file.
        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'article.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'article.tex'], stderr = STDOUT)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Mathematics of Computation (MCOM).
# https://www.ams.org/publications/journals/journalsframework/mcomauthorpac

import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.ams.org/arc/journals/packages/mcom/mcom_amslatex.zip', junk = 1)

        # Compile the template file.
        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'mcom-l-template.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'mcom-l-template.tex'], stderr = STDOUT)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# SIAM online journals.
# https://epubs.siam.org/journal-authors

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://epubs.siam.org/pb-assets/macros/online/siamonline_250106.zip', method = "wget")

        # Get the separate style manual as well.
        utilities.get_file('https://epubs.siam.org/pb-assets/files/SIAM_STYLE_GUIDE_2019.pdf', method = "wget")

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Taylor and Francis journals.
# https://www.tandfonline.com/action/authorSubmission?show=instructions\&journalCode=gopt20

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://files.taylorandfrancis.com/InteractNLMLaTeX.zip')

        # Get the separate style manual as well.
        utilities.get_file('https://files.taylorandfrancis.com/tf_NLM.pdf')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Springer journals based on the svjour3.cls document class.
# https://www.springer.com/journal/245/submission-guidelines

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://media.springer.com/full/springer-instructions-for-authors-assets/zip/468198_LaTeX_DL_468198_240419.zip', junk = 1)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Journal of Nonsmooth Analysis and Optimization (JNSAO).
# https://jnsao.episciences.org/page/for-authors

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://jnsao.episciences.org/public/jnsao_latex.zip')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Elsevier journals.
# https://www.elsevier.com/authors/author-schemas/latex-instructions

import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://mirrors.ctan.org/macros/latex/contrib/elsarticle.zip', junk = 1)

        # Run the installer, which is going to create the elsarticle.cls file.
        run(['latex', 'elsarticle.ins'], stderr = STDOUT)

        # Compile the template files.
        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'elsarticle-template-harv.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'elsarticle-template-harv.tex'], stderr = STDOUT)

        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'elsarticle-template-num.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'elsarticle-template-num.tex'], stderr = STDOUT)

        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'elsarticle-template-num-names.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'elsarticle-template-num-names.tex'], stderr = STDOUT)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



"""
    pygments.lexers.markup
    ~~~~~~~~~~~~~~~~~~~~~~

    Lexers for non-HTML markup languages.

    :copyright: Copyright 2006-2022 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import re

from pygments.lexers.html import XmlLexer
from pygments.lexers.javascript import JavascriptLexer
from pygments.lexers.css import CssLexer

from pygments.lexer import RegexLexer, DelegatingLexer, include, bygroups, \
    using, this, do_insertions, default, words
from pygments.token import Text, Comment, Operator, Keyword, Name, String, \
    Number, Punctuation, Generic, Other, Whitespace
from pygments.util import get_bool_opt, ClassNotFound

__all__ = ['BBCodeLexer', 'MoinWikiLexer', 'RstLexer', 'TexLexer', 'GroffLexer',
           'MozPreprocHashLexer', 'MozPreprocPercentLexer',
           'MozPreprocXulLexer', 'MozPreprocJavascriptLexer',
           'MozPreprocCssLexer', 'MarkdownLexer', 'TiddlyWiki5Lexer']


class BBCodeLexer(RegexLexer):
    """
    A lexer that highlights BBCode(-like) syntax.

    .. versionadded:: 0.6
    """

    name = 'BBCode'
    aliases = ['bbcode']
    mimetypes = ['text/x-bbcode']

    tokens = {
        'root': [
            (r'[^[]+', Text),
            # tag/end tag begin
            (r'\[/?\w+', Keyword, 'tag'),
            # stray bracket
            (r'\[', Text),
        ],
        'tag': [
            (r'\s+', Text),
            # attribute with value
            (r'(\w+)(=)("?[^\s"\]]+"?)',
             bygroups(Name.Attribute, Operator, String)),
            # tag argument (a la [color=green])
            (r'(=)("?[^\s"\]]+"?)',
             bygroups(Operator, String)),
            # tag end
            (r'\]', Keyword, '#pop'),
        ],
    }


class MoinWikiLexer(RegexLexer):
    """
    For MoinMoin (and Trac) Wiki markup.

    .. versionadded:: 0.7
    """

    name = 'MoinMoin/Trac Wiki markup'
    aliases = ['trac-wiki', 'moin']
    filenames = []
    mimetypes = ['text/x-trac-wiki']
    flags = re.MULTILINE | re.IGNORECASE

    tokens = {
        'root': [
            (r'^#.*$', Comment),
            (r'(!)(\S+)', bygroups(Keyword, Text)),  # Ignore-next
            # Titles
            (r'^(=+)([^=]+)(=+)(\s*#.+)?$',
             bygroups(Generic.Heading, using(this), Generic.Heading, String)),
            # Literal code blocks, with optional shebang
            (r'(\{\{\{)(\n#!.+)?', bygroups(Name.Builtin, Name.Namespace), 'codeblock'),
            (r'(\'\'\'?|\|\||`|__|~~|\^|,,|::)', Comment),  # Formatting
            # Lists
            (r'^( +)([.*-])( )', bygroups(Text, Name.Builtin, Text)),
            (r'^( +)([a-z]{1,5}\.)( )', bygroups(Text, Name.Builtin, Text)),
            # Other Formatting
            (r'\[\[\w+.*?\]\]', Keyword),  # Macro
            (r'(\[[^\s\]]+)(\s+[^\]]+?)?(\])',
             bygroups(Keyword, String, Keyword)),  # Link
            (r'^----+$', Keyword),  # Horizontal rules
            (r'[^\n\'\[{!_~^,|]+', Text),
            (r'\n', Text),
            (r'.', Text),
        ],
        'codeblock': [
            (r'\}\}\}', Name.Builtin, '#pop'),
            # these blocks are allowed to be nested in Trac, but not MoinMoin
            (r'\{\{\{', Text, '#push'),
            (r'[^{}]+', Comment.Preproc),  # slurp boring text
            (r'.', Comment.Preproc),  # allow loose { or }
        ],
    }


class RstLexer(RegexLexer):
    """
    For reStructuredText markup.

    .. versionadded:: 0.7

    Additional options accepted:

    `handlecodeblocks`
        Highlight the contents of ``.. sourcecode:: language``,
        ``.. code:: language`` and ``.. code-block:: language``
        directives with a lexer for the given language (default:
        ``True``).

        .. versionadded:: 0.8
    """
    name = 'reStructuredText'
    url = 'https://docutils.sourceforge.io/rst.html'
    aliases = ['restructuredtext', 'rst', 'rest']
    filenames = ['*.rst', '*.rest']
    mimetypes = ["text/x-rst", "text/prs.fallenstein.rst"]
    flags = re.MULTILINE

    def _handle_sourcecode(self, match):
        from pygments.lexers import get_lexer_by_name

        # section header
        yield match.start(1), Punctuation, match.group(1)
        yield match.start(2), Text, match.group(2)
        yield match.start(3), Operator.Word, match.group(3)
        yield match.start(4), Punctuation, match.group(4)
        yield match.start(5), Text, match.group(5)
        yield match.start(6), Keyword, match.group(6)
        yield match.start(7), Text, match.group(7)

        # lookup lexer if wanted and existing
        lexer = None
        if self.handlecodeblocks:
            try:
                lexer = get_lexer_by_name(match.group(6).strip())
            except ClassNotFound:
                pass
        indention = match.group(8)
        indention_size = len(indention)
        code = (indention + match.group(9) + match.group(10) + match.group(11))

        # no lexer for this language. handle it like it was a code block
        if lexer is None:
            yield match.start(8), String, code
            return

        # highlight the lines with the lexer.
        ins = []
        codelines = code.splitlines(True)
        code = ''
        for line in codelines:
            if len(line) > indention_size:
                ins.append((len(code), [(0, Text, line[:indention_size])]))
                code += line[indention_size:]
            else:
                code += line
        yield from do_insertions(ins, lexer.get_tokens_unprocessed(code))

    # from docutils.parsers.rst.states
    closers = '\'")]}>\u2019\u201d\xbb!?'
    unicode_delimiters = '\u2010\u2011\u2012\u2013\u2014\u00a0'
    end_string_suffix = (r'((?=$)|(?=[-/:.,; \n\x00%s%s]))'
                         % (re.escape(unicode_delimiters),
                            re.escape(closers)))

    tokens = {
        'root': [
            # Heading with overline
            (r'^(=+|-+|`+|:+|\.+|\'+|"+|~+|\^+|_+|\*+|\++|#+)([ \t]*\n)'
             r'(.+)(\n)(\1)(\n)',
             bygroups(Generic.Heading, Text, Generic.Heading,
                      Text, Generic.Heading, Text)),
            # Plain heading
            (r'^(\S.*)(\n)(={3,}|-{3,}|`{3,}|:{3,}|\.{3,}|\'{3,}|"{3,}|'
             r'~{3,}|\^{3,}|_{3,}|\*{3,}|\+{3,}|#{3,})(\n)',
             bygroups(Generic.Heading, Text, Generic.Heading, Text)),
            # Bulleted lists
            (r'^(\s*)([-*+])( .+\n(?:\1  .+\n)*)',
             bygroups(Text, Number, using(this, state='inline'))),
            # Numbered lists
            (r'^(\s*)([0-9#ivxlcmIVXLCM]+\.)( .+\n(?:\1  .+\n)*)',
             bygroups(Text, Number, using(this, state='inline'))),
            (r'^(\s*)(\(?[0-9#ivxlcmIVXLCM]+\))( .+\n(?:\1  .+\n)*)',
             bygroups(Text, Number, using(this, state='inline'))),
            # Numbered, but keep words at BOL from becoming lists
            (r'^(\s*)([A-Z]+\.)( .+\n(?:\1  .+\n)+)',
             bygroups(Text, Number, using(this, state='inline'))),
            (r'^(\s*)(\(?[A-Za-z]+\))( .+\n(?:\1  .+\n)+)',
             bygroups(Text, Number, using(this, state='inline'))),
            # Line blocks
            (r'^(\s*)(\|)( .+\n(?:\|  .+\n)*)',
             bygroups(Text, Operator, using(this, state='inline'))),
            # Sourcecode directives
            (r'^( *\.\.)(\s*)((?:source)?code(?:-block)?)(::)([ \t]*)([^\n]+)'
             r'(\n[ \t]*\n)([ \t]+)(.*)(\n)((?:(?:\8.*)?\n)+)',
             _handle_sourcecode),
            # A directive
            (r'^( *\.\.)(\s*)([\w:-]+?)(::)(?:([ \t]*)(.*))',
             bygroups(Punctuation, Text, Operator.Word, Punctuation, Text,
                      using(this, state='inline'))),
            # A reference target
            (r'^( *\.\.)(\s*)(_(?:[^:\\]|\\.)+:)(.*?)$',
             bygroups(Punctuation, Text, Name.Tag, using(this, state='inline'))),
            # A footnote/citation target
            (r'^( *\.\.)(\s*)(\[.+\])(.*?)$',
             bygroups(Punctuation, Text, Name.Tag, using(this, state='inline'))),
            # A substitution def
            (r'^( *\.\.)(\s*)(\|.+\|)(\s*)([\w:-]+?)(::)(?:([ \t]*)(.*))',
             bygroups(Punctuation, Text, Name.Tag, Text, Operator.Word,
                      Punctuation, Text, using(this, state='inline'))),
            # Comments
            (r'^ *\.\..*(\n( +.*\n|\n)+)?', Comment.Preproc),
            # Field list marker
            (r'^( *)(:(?:\\\\|\\:|[^:\n])+:(?=\s))([ \t]*)',
             bygroups(Text, Name.Class, Text)),
            # Definition list
            (r'^(\S.*(?<!::)\n)((?:(?: +.*)\n)+)',
             bygroups(using(this, state='inline'), using(this, state='inline'))),
            # Code blocks
            (r'(::)(\n[ \t]*\n)([ \t]+)(.*)(\n)((?:(?:\3.*)?\n)+)',
             bygroups(String.Escape, Text, String, String, Text, String)),
            include('inline'),
        ],
        'inline': [
            (r'\\.', Text),  # escape
            (r'``', String, 'literal'),  # code
            (r'(`.+?)(<.+?>)(`__?)',  # reference with inline target
             bygroups(String, String.Interpol, String)),
            (r'`.+?`__?', String),  # reference
            (r'(`.+?`)(:[a-zA-Z0-9:-]+?:)?',
             bygroups(Name.Variable, Name.Attribute)),  # role
            (r'(:[a-zA-Z0-9:-]+?:)(`.+?`)',
             bygroups(Name.Attribute, Name.Variable)),  # role (content first)
            (r'\*\*.+?\*\*', Generic.Strong),  # Strong emphasis
            (r'\*.+?\*', Generic.Emph),  # Emphasis
            (r'\[.*?\]_', String),  # Footnote or citation
            (r'<.+?>', Name.Tag),   # Hyperlink
            (r'[^\\\n\[*`:]+', Text),
            (r'.', Text),
        ],
        'literal': [
            (r'[^`]+', String),
            (r'``' + end_string_suffix, String, '#pop'),
            (r'`', String),
        ]
    }

    def __init__(self, **options):
        self.handlecodeblocks = get_bool_opt(options, 'handlecodeblocks', True)
        RegexLexer.__init__(self, **options)

    def analyse_text(text):
        if text[:2] == '..' and text[2:3] != '.':
            return 0.3
        p1 = text.find("\n")
        p2 = text.find("\n", p1 + 1)
        if (p2 > -1 and              # has two lines
                p1 * 2 + 1 == p2 and     # they are the same length
                text[p1+1] in '-=' and   # the next line both starts and ends with
                text[p1+1] == text[p2-1]):  # ...a sufficiently high header
            return 0.5


class TexLexer(RegexLexer):
    """
    Lexer for the TeX and LaTeX typesetting languages.
    """

    name = 'TeX'
    aliases = ['tex', 'latex']
    filenames = ['*.tex', '*.aux', '*.toc']
    mimetypes = ['text/x-tex', 'text/x-latex']

    tokens = {
        'general': [
            (r'%.*?\n', Comment),
            (r'[{}]', Name.Builtin),
            (r'[&_^]', Name.Builtin),
        ],
        'root': [
            (r'\\\[', String.Backtick, 'displaymath'),
            (r'\\\(', String, 'inlinemath'),
            (r'\$\$', String.Backtick, 'displaymath'),
            (r'\$', String, 'inlinemath'),
            (r'\\([a-zA-Z@]+|.)', Keyword, 'command'),
            (r'\\$', Keyword),
            include('general'),
            (r'[^\\$%&_^{}]+', Text),
        ],
        'math': [
            (r'\\([a-zA-Z@]+|.)', Name.Variable),
            include('general'),
            (r'[0-9]+', Number),
            (r'[-=!+*/()\[\]]', Operator),
            (r'[^=!+*/()\[\]\\$%&_^{}0-9-]+', Name.Builtin),
        ],
        'inlinemath': [
            (r'\\\)', String, '#pop'),
            (r'\$', String, '#pop'),
            include('math'),
        ],
        'displaymath': [
            (r'\\\]', String, '#pop'),
            (r'\$\$', String, '#pop'),
            (r'\$', Name.Builtin),
            include('math'),
        ],
        'command': [
            (r'\[.*?\]', Name.Attribute),
            (r'\*', Keyword),
            default('#pop'),
        ],
    }

    def analyse_text(text):
        for start in ("\\documentclass", "\\input", "\\documentstyle",
                      "\\relax"):
            if text[:len(start)] == start:
                return True


class GroffLexer(RegexLexer):
    """
    Lexer for the (g)roff typesetting language, supporting groff
    extensions. Mainly useful for highlighting manpage sources.

    .. versionadded:: 0.6
    """

    name = 'Groff'
    aliases = ['groff', 'nroff', 'man']
    filenames = ['*.[1-9]', '*.man', '*.1p', '*.3pm']
    mimetypes = ['application/x-troff', 'text/troff']

    tokens = {
        'root': [
            (r'(\.)(\w+)', bygroups(Text, Keyword), 'request'),
            (r'\.', Punctuation, 'request'),
            # Regular characters, slurp till we find a backslash or newline
            (r'[^\\\n]+', Text, 'textline'),
            default('textline'),
        ],
        'textline': [
            include('escapes'),
            (r'[^\\\n]+', Text),
            (r'\n', Text, '#pop'),
        ],
        'escapes': [
            # groff has many ways to write escapes.
            (r'\\"[^\n]*', Comment),
            (r'\\[fn]\w', String.Escape),
            (r'\\\(.{2}', String.Escape),
            (r'\\.\[.*\]', String.Escape),
            (r'\\.', String.Escape),
            (r'\\\n', Text, 'request'),
        ],
        'request': [
            (r'\n', Text, '#pop'),
            include('escapes'),
            (r'"[^\n"]+"', String.Double),
            (r'\d+', Number),
            (r'\S+', String),
            (r'\s+', Text),
        ],
    }

    def analyse_text(text):
        if text[:1] != '.':
            return False
        if text[:3] == '.\\"':
            return True
        if text[:4] == '.TH ':
            return True
        if text[1:3].isalnum() and text[3].isspace():
            return 0.9


class MozPreprocHashLexer(RegexLexer):
    """
    Lexer for Mozilla Preprocessor files (with '#' as the marker).

    Other data is left untouched.

    .. versionadded:: 2.0
    """
    name = 'mozhashpreproc'
    aliases = [name]
    filenames = []
    mimetypes = []

    tokens = {
        'root': [
            (r'^#', Comment.Preproc, ('expr', 'exprstart')),
            (r'.+', Other),
        ],
        'exprstart': [
            (r'(literal)(.*)', bygroups(Comment.Preproc, Text), '#pop:2'),
            (words((
                'define', 'undef', 'if', 'ifdef', 'ifndef', 'else', 'elif',
                'elifdef', 'elifndef', 'endif', 'expand', 'filter', 'unfilter',
                'include', 'includesubst', 'error')),
             Comment.Preproc, '#pop'),
        ],
        'expr': [
            (words(('!', '!=', '==', '&&', '||')), Operator),
            (r'(defined)(\()', bygroups(Keyword, Punctuation)),
            (r'\)', Punctuation),
            (r'[0-9]+', Number.Decimal),
            (r'__\w+?__', Name.Variable),
            (r'@\w+?@', Name.Class),
            (r'\w+', Name),
            (r'\n', Text, '#pop'),
            (r'\s+', Text),
            (r'\S', Punctuation),
        ],
    }


class MozPreprocPercentLexer(MozPreprocHashLexer):
    """
    Lexer for Mozilla Preprocessor files (with '%' as the marker).

    Other data is left untouched.

    .. versionadded:: 2.0
    """
    name = 'mozpercentpreproc'
    aliases = [name]
    filenames = []
    mimetypes = []

    tokens = {
        'root': [
            (r'^%', Comment.Preproc, ('expr', 'exprstart')),
            (r'.+', Other),
        ],
    }


class MozPreprocXulLexer(DelegatingLexer):
    """
    Subclass of the `MozPreprocHashLexer` that highlights unlexed data with the
    `XmlLexer`.

    .. versionadded:: 2.0
    """
    name = "XUL+mozpreproc"
    aliases = ['xul+mozpreproc']
    filenames = ['*.xul.in']
    mimetypes = []

    def __init__(self, **options):
        super().__init__(XmlLexer, MozPreprocHashLexer, **options)


class MozPreprocJavascriptLexer(DelegatingLexer):
    """
    Subclass of the `MozPreprocHashLexer` that highlights unlexed data with the
    `JavascriptLexer`.

    .. versionadded:: 2.0
    """
    name = "Javascript+mozpreproc"
    aliases = ['javascript+mozpreproc']
    filenames = ['*.js.in']
    mimetypes = []

    def __init__(self, **options):
        super().__init__(JavascriptLexer, MozPreprocHashLexer, **options)


class MozPreprocCssLexer(DelegatingLexer):
    """
    Subclass of the `MozPreprocHashLexer` that highlights unlexed data with the
    `CssLexer`.

    .. versionadded:: 2.0
    """
    name = "CSS+mozpreproc"
    aliases = ['css+mozpreproc']
    filenames = ['*.css.in']
    mimetypes = []

    def __init__(self, **options):
        super().__init__(CssLexer, MozPreprocPercentLexer, **options)


class MarkdownLexer(RegexLexer):
    """
    For Markdown markup.

    .. versionadded:: 2.2
    """
    name = 'Markdown'
    url = 'https://daringfireball.net/projects/markdown/'
    aliases = ['markdown', 'md']
    filenames = ['*.md', '*.markdown']
    mimetypes = ["text/x-markdown"]
    flags = re.MULTILINE

    def _handle_codeblock(self, match):
        """
        match args: 1:backticks, 2:lang_name, 3:newline, 4:code, 5:backticks
        """
        from pygments.lexers import get_lexer_by_name

        # section header
        yield match.start(1), String.Backtick, match.group(1)
        yield match.start(2), String.Backtick, match.group(2)
        yield match.start(3), Text           , match.group(3)

        # lookup lexer if wanted and existing
        lexer = None
        if self.handlecodeblocks:
            try:
                lexer = get_lexer_by_name( match.group(2).strip() )
            except ClassNotFound:
                pass
        code = match.group(4)

        # no lexer for this language. handle it like it was a code block
        if lexer is None:
            yield match.start(4), String, code
        else:
            yield from do_insertions([], lexer.get_tokens_unprocessed(code))

        yield match.start(5), String.Backtick, match.group(5)

    tokens = {
        'root': [
            # heading with '#' prefix (atx-style)
            (r'(^#[^#].+)(\n)', bygroups(Generic.Heading, Text)),
            # subheading with '#' prefix (atx-style)
            (r'(^#{2,6}[^#].+)(\n)', bygroups(Generic.Subheading, Text)),
            # heading with '=' underlines (Setext-style)
            (r'^(.+)(\n)(=+)(\n)', bygroups(Generic.Heading, Text, Generic.Heading, Text)),
            # subheading with '-' underlines (Setext-style)
            (r'^(.+)(\n)(-+)(\n)', bygroups(Generic.Subheading, Text, Generic.Subheading, Text)),
            # task list
            (r'^(\s*)([*-] )(\[[ xX]\])( .+\n)',
            bygroups(Whitespace, Keyword, Keyword, using(this, state='inline'))),
            # bulleted list
            (r'^(\s*)([*-])(\s)(.+\n)',
            bygroups(Whitespace, Keyword, Whitespace, using(this, state='inline'))),
            # numbered list
            (r'^(\s*)([0-9]+\.)( .+\n)',
            bygroups(Whitespace, Keyword, using(this, state='inline'))),
            # quote
            (r'^(\s*>\s)(.+\n)', bygroups(Keyword, Generic.Emph)),
            # code block fenced by 3 backticks
            (r'^(\s*```\n[\w\W]*?^\s*```$\n)', String.Backtick),
            # code block with language
            (r'^(\s*```)(\w+)(\n)([\w\W]*?)(^\s*```$\n)', _handle_codeblock),

            include('inline'),
        ],
        'inline': [
            # escape
            (r'\\.', Text),
            # inline code
            (r'([^`]?)(`[^`\n]+`)', bygroups(Text, String.Backtick)),
            # warning: the following rules eat outer tags.
            # eg. **foo _bar_ baz** => foo and baz are not recognized as bold
            # bold fenced by '**'
            (r'([^\*]?)(\*\*[^* \n][^*\n]*\*\*)', bygroups(Text, Generic.Strong)),
            # bold fenced by '__'
            (r'([^_]?)(__[^_ \n][^_\n]*__)', bygroups(Text, Generic.Strong)),
            # italics fenced by '*'
            (r'([^\*]?)(\*[^* \n][^*\n]*\*)', bygroups(Text, Generic.Emph)),
            # italics fenced by '_'
            (r'([^_]?)(_[^_ \n][^_\n]*_)', bygroups(Text, Generic.Emph)),
            # strikethrough
            (r'([^~]?)(~~[^~ \n][^~\n]*~~)', bygroups(Text, Generic.Deleted)),
            # mentions and topics (twitter and github stuff)
            (r'[@#][\w/:]+', Name.Entity),
            # (image?) links eg: ![Image of Yaktocat](https://octodex.github.com/images/yaktocat.png)
            (r'(!?\[)([^]]+)(\])(\()([^)]+)(\))',
             bygroups(Text, Name.Tag, Text, Text, Name.Attribute, Text)),
            # reference-style links, e.g.:
            #   [an example][id]
            #   [id]: http://example.com/
            (r'(\[)([^]]+)(\])(\[)([^]]*)(\])',
             bygroups(Text, Name.Tag, Text, Text, Name.Label, Text)),
            (r'^(\s*\[)([^]]*)(\]:\s*)(.+)',
             bygroups(Text, Name.Label, Text, Name.Attribute)),

            # general text, must come last!
            (r'[^\\\s]+', Text),
            (r'.', Text),
        ],
    }

    def __init__(self, **options):
        self.handlecodeblocks = get_bool_opt(options, 'handlecodeblocks', True)
        RegexLexer.__init__(self, **options)


class TiddlyWiki5Lexer(RegexLexer):
    """
    For TiddlyWiki5 markup.

    .. versionadded:: 2.7
    """
    name = 'tiddler'
    url = 'https://tiddlywiki.com/#TiddlerFiles'
    aliases = ['tid']
    filenames = ['*.tid']
    mimetypes = ["text/vnd.tiddlywiki"]
    flags = re.MULTILINE

    def _handle_codeblock(self, match):
        """
        match args: 1:backticks, 2:lang_name, 3:newline, 4:code, 5:backticks
        """
        from pygments.lexers import get_lexer_by_name

        # section header
        yield match.start(1), String, match.group(1)
        yield match.start(2), String, match.group(2)
        yield match.start(3), Text,   match.group(3)

        # lookup lexer if wanted and existing
        lexer = None
        if self.handlecodeblocks:
            try:
                lexer = get_lexer_by_name(match.group(2).strip())
            except ClassNotFound:
                pass
        code = match.group(4)

        # no lexer for this language. handle it like it was a code block
        if lexer is None:
            yield match.start(4), String, code
            return

        yield from do_insertions([], lexer.get_tokens_unprocessed(code))

        yield match.start(5), String, match.group(5)

    def _handle_cssblock(self, match):
        """
        match args: 1:style tag 2:newline, 3:code, 4:closing style tag
        """
        from pygments.lexers import get_lexer_by_name

        # section header
        yield match.start(1), String, match.group(1)
        yield match.start(2), String, match.group(2)

        lexer = None
        if self.handlecodeblocks:
            try:
                lexer = get_lexer_by_name('css')
            except ClassNotFound:
                pass
        code = match.group(3)

        # no lexer for this language. handle it like it was a code block
        if lexer is None:
            yield match.start(3), String, code
            return

        yield from do_insertions([], lexer.get_tokens_unprocessed(code))

        yield match.start(4), String, match.group(4)

    tokens = {
        'root': [
            # title in metadata section
            (r'^(title)(:\s)(.+\n)', bygroups(Keyword, Text, Generic.Heading)),
            # headings
            (r'^(!)([^!].+\n)', bygroups(Generic.Heading, Text)),
            (r'^(!{2,6})(.+\n)', bygroups(Generic.Subheading, Text)),
            # bulleted or numbered lists or single-line block quotes
            # (can be mixed)
            (r'^(\s*)([*#>]+)(\s*)(.+\n)',
             bygroups(Text, Keyword, Text, using(this, state='inline'))),
            # multi-line block quotes
            (r'^(<<<.*\n)([\w\W]*?)(^<<<.*$)', bygroups(String, Text, String)),
            # table header
            (r'^(\|.*?\|h)$', bygroups(Generic.Strong)),
            # table footer or caption
            (r'^(\|.*?\|[cf])$', bygroups(Generic.Emph)),
            # table class
            (r'^(\|.*?\|k)$', bygroups(Name.Tag)),
            # definitions
            (r'^(;.*)$', bygroups(Generic.Strong)),
            # text block
            (r'^(```\n)([\w\W]*?)(^```$)', bygroups(String, Text, String)),
            # code block with language
            (r'^(```)(\w+)(\n)([\w\W]*?)(^```$)', _handle_codeblock),
            # CSS style block
            (r'^(<style>)(\n)([\w\W]*?)(^</style>$)', _handle_cssblock),

            include('keywords'),
            include('inline'),
        ],
        'keywords': [
            (words((
                '\\define', '\\end', 'caption', 'created', 'modified', 'tags',
                'title', 'type'), prefix=r'^', suffix=r'\b'),
             Keyword),
        ],
        'inline': [
            # escape
            (r'\\.', Text),
            # created or modified date
            (r'\d{17}', Number.Integer),
            # italics
            (r'(\s)(//[^/]+//)((?=\W|\n))',
             bygroups(Text, Generic.Emph, Text)),
            # superscript
            (r'(\s)(\^\^[^\^]+\^\^)', bygroups(Text, Generic.Emph)),
            # subscript
            (r'(\s)(,,[^,]+,,)', bygroups(Text, Generic.Emph)),
            # underscore
            (r'(\s)(__[^_]+__)', bygroups(Text, Generic.Strong)),
            # bold
            (r"(\s)(''[^']+'')((?=\W|\n))",
             bygroups(Text, Generic.Strong, Text)),
            # strikethrough
            (r'(\s)(~~[^~]+~~)((?=\W|\n))',
             bygroups(Text, Generic.Deleted, Text)),
            # TiddlyWiki variables
            (r'<<[^>]+>>', Name.Tag),
            (r'\$\$[^$]+\$\$', Name.Tag),
            (r'\$\([^)]+\)\$', Name.Tag),
            # TiddlyWiki style or class
            (r'^@@.*$', Name.Tag),
            # HTML tags
            (r'</?[^>]+>', Name.Tag),
            # inline code
            (r'`[^`]+`', String.Backtick),
            # HTML escaped symbols
            (r'&\S*?;', String.Regex),
            # Wiki links
            (r'(\[{2})([^]\|]+)(\]{2})', bygroups(Text, Name.Tag, Text)),
            # External links
            (r'(\[{2})([^]\|]+)(\|)([^]\|]+)(\]{2})',
            bygroups(Text, Name.Tag, Text, Name.Attribute, Text)),
            # Transclusion
            (r'(\{{2})([^}]+)(\}{2})', bygroups(Text, Name.Tag, Text)),
            # URLs
            (r'(\b.?.?tps?://[^\s"]+)', bygroups(Name.Attribute)),

            # general text, must come last!
            (r'[\w]+', Text),
            (r'.', Text)
        ],
    }

    def __init__(self, **options):
        self.handlecodeblocks = get_bool_opt(options, 'handlecodeblocks', True)
        RegexLexer.__init__(self, **options)



# This script initializes the LaTeX author resources provided for
# Proceedings in Applied Mathematics and Mechanics (PAMM).
# https://onlinelibrary.wiley.com/page/journal/16177061/homepage/2130_authorguidelines.html

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.wiley-vch.de/vch/journals/2130/public/pamm_latex_unix.tgz', junk = 1)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# LaPreprint.
# https://github.com/LaPreprint/LaPreprint

import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get the LaTeX author resources from the publisher.
        utilities.get_archive('https://github.com/LaPreprint/LaPreprint/archive/refs/heads/main.zip', junk = 1)

        # Compile the template file.
        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'main.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'main.tex'], stderr = STDOUT)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Parallel Processing Letters (PPL).
# https://www.worldscientific.com/page/authors/journal-stylefiles

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.worldscientific.com/sda/1037/ppl-2e.zip')

        # Get the separate style manual as well.
        utilities.get_file('https://www.worldscientific.com/sda/1037/ws-ppl.pdf')

        # The file ws-ppl.bst has some issues.
        # One issue is that a @BOOK entry requires the VOLUME field to be nonempty whenever the SERIES field is nonempty.
        # Another issue is that @PHDTHESIS without an ADDRESS field but with a YEAR field yields an error.
        # We apply a patch created via
        #   diff -u ws-ppl.bst ws-ppl-scoop.bst > ws-ppl.bst.patch
        utilities.apply_patch('ws-ppl.bst.patch')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# Journal of Inverse and Ill-posed Problems (JIP).
# https://www.degruyter.com/journal/key/jiip/html

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.degruyter.com/publication/journal_key/JIIP/downloadAsset/JIIP_LaTeX-Template-for-Authors.zip')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# BIT Numerical Mathematics (BIT).
# https://projects.mai.liu.se/BIT/Contributions.html

import os
import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get the LaTeX author resources from the publisher.
        utilities.get_file('https://projects.mai.liu.se/BIT/BITtemplate.tex')
        utilities.get_file('https://projects.mai.liu.se/BIT/svjour3.cls')
        utilities.get_file('https://projects.mai.liu.se/BIT/svglov3.clo')
        utilities.get_file('https://projects.mai.liu.se/BIT/usrguid3.pdf')

        # Link to spmpsci.bst which is recommended but not available from the author resources page.
        utilities.link_or_copy('../svjour3.cls/spmpsci.bst', 'spmpsci.bst')

        # Compile the template file.
        run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'BITtemplate.tex'], stderr = STDOUT)
        run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'BITtemplate.tex'], stderr = STDOUT)

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This class introduces a parser object intended for templates of type letters.

# Resolve the dependencies.
import yaml

class parserObject(object):

    def __init__(self, parserInfo):
        # Initialize the data structure.
        self.latex = parserInfo.dataFileData.get("latex", {}) or {}
        self.letter = parserInfo.dataFileData.get("letter", {}) or {}
        self.outDirectory = parserInfo.outDirectory
        self.outFileBaseName = parserInfo.outFileBaseName
        self.templateBaseName = parserInfo.templateBaseName
        self.templateDescription = parserInfo.templateDescription
        self.scoopTemplateEngineVersion = parserInfo.scoopTemplateEngineVersion
        self.thisScriptAbsolutePathCallSummary = parserInfo.thisScriptAbsolutePathCallSummary
        self.dataFileData = parserInfo.dataFileData


    # LaTeXPrePreamble()
    # returns a string such as
    #   % Insert the user-defined prepreamble.
    #   \latex["prepreamble"] as is with "" as default.
    # It is intended to be included before \documentclass.
    def LaTeXPrePreamble(self):
        prePreambleString = """\
% Insert the user-defined prepreamble.
{prePreamble:s}\
    """.format(prePreamble = self.latex.get("prepreamble", "") or ""
            )
        return prePreambleString

    # LaTeXPreamble()
    # returns a string such as
    #   % Insert the user-defined preamble.
    #   \latex["preamble"] as is with "" as default
    # It is intended to be included after \documentclass.
    def LaTeXPreamble(self):
        preambleString = """\
% Insert the user-defined preamble.
{preamble:s}\
""".format(preamble = self.latex.get("preamble", "") or "",
            )
        return preambleString

    # LaTeXBibFiles()
    # returns latex["bibfiles"] as a list of strings
    # with "None"s eliminated.
    def LaTeXBibFiles(self):
        content = self.EnsureList(self.latex.get("bibfiles", "") or [])
        return [item for item in content if item]

    # LaTeXBody()
    # returns latex["body"] as a list of strings
    # with "None"s eliminated.
    def LaTeXBody(self):
        content = self.EnsureList(self.latex.get("body", "") or [])
        return [item for item in content if item]

    # DocumentClassOptions()
    # returns a comma separated string.
    # It is intended to be used as document class options in \documentclass.
    def DocumentClassOptions(self):
        content = self.EnsureList(self.latex.get("documentclassoptions", "") or [])
        return ",".join([item for item in content if item])

    # EnsureList(content)
    # returns content as is, if it already is a list, or [content] if not.
    def EnsureList(self, content):
        if isinstance(content, list):
            return content
        else:
            return [content]

    # CommandWrapper(content, command, separator)
    # where content is a string or a list of strings
    # and command is a string or a list of strings
    # returns a string such as
    #   \command{content}
    # .
    # When content == ["item1", "item2", "item3"] and
    # command == ["command1", "command2"],
    # then it returns
    #   \command1{item1}
    #   \command2{item2}
    #   \command2{item3}
    # .
    # The default separator, which is used to join the list items, is "\n".
    # When the separator is 'None', then no concatenation will be performed.
    def CommandWrapper(self, content, command, separator = "\n"):
        content = self.EnsureList(content)
        commands = self.EnsureList(command)
        commands = commands[:len(content)]  # truncate the commands list if necessary
        commands.extend(commands[-1:] * (len(content) - len(commands)))  # amend the commands list by repeating the last element if necessary
        commands = [command + "{" + item + "}" for (command, item) in zip(commands, content)]
        if separator is not None:
            return separator.join(commands)
        else:
            return commands

    # BibLaTeXResources()
    # returns a string such as
    #   \addbibresources{bibfile.bib}{}
    #   \addbibresources{bibfile.bib}{}
    # .
    def BibLaTeXResources(self):
        bibFiles = self.LaTeXBibFiles()
        return "\n".join([self.CommandWrapper(bibFile, "\\addbibresource") for bibFile in bibFiles])

    # BibLaTeXPrintBibliography()
    # returns a string such as
    #   \printbibliography
    def BibLaTeXPrintBibliography(self):
        return "\\printbibliography"

    # InputBody()
    # returns a string such as
    #   \input{file.tex}
    #   \input{file.tex}
    # .
    def InputBody(self):
        body = self.LaTeXBody()
        return "\n".join(["\\input{" + bodyFile + "}" for bodyFile in body])

    # Recipient()
    # returns the post scriptum string.
    def Recipient(self):
        return self.letter.get("recipient", "~") or "~"

    # CC()
    # returns a string such as
    #   \cc{Recipient \\ Recipient}
    # .
    def CC(self):
        content = self.letter.get("cc", "") or ""
        if content:
            return self.CommandWrapper("\\\\".join(self.EnsureList(content)), "\\cc")
        else:
            return content

    # Encl()
    # returns a string such as
    #   \encl{Document \\ Document}
    # .
    def Encl(self):
        content = self.letter.get("encl", "") or ""
        if content:
            return self.CommandWrapper("\\\\".join(self.EnsureList(content)), "\\encl")
        else:
            return content

    # PS()
    # returns the post scriptum string.
    def PS(self):
        return self.letter.get("ps", "") or ""

    # Subject()
    # returns the subject string.
    def Subject(self):
        return self.letter.get("subject", "") or ""

    # Date()
    # returns the date string.
    def Date(self):
        return self.letter.get("date", "") or ""

    # ScoopTemplateEngineSignature()
    # returns a signature containing the scoopTemplateEngineVersion.
    def ScoopTemplateEngineSignature(self):
        return "Created using the Scoop Template Engine version " + self.scoopTemplateEngineVersion + "."

    # TemplateTitle()
    # returns the template description as is.
    def TemplateTitle(self):
        return self.templateDescription




# This script initializes the LaTeX author resources provided for
# IMA Journal of Numerical Analysis.
# https://academic.oup.com/imajna/pages/General_Instructions

import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://static.primary.prod.gcms.the-infra.com/static/site/imamat/document/ima-authoring-template.zip?node=ad2e6afe141e2df8c873')

        # Get the separate style manual as well.
        utilities.get_file('https://static.primary.prod.gcms.the-infra.com/static/site/imajna/document/styleguide.pdf?node=5b8886f8fcd3ab63fda8')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)



# This script initializes the LaTeX author resources provided for
# International Journal for Uncertainty Quantification (IJUQ).
# https://www.submission.begellhouse.com/help/hub.html

import os
import sys
from ste.utilities import utilities

if __name__ == '__main__':
    try:
        # Remove the initialization time and version stamp.
        utilities.remove_time_version_stamp()

        # Get and unpack the LaTeX author resources from the publisher.
        utilities.get_archive('https://www.submission.begellhouse.com/download/latex_template/52034eb04b657aea/IJUQ_Latex_Template.zip')

        # Get the non-standard uarial.sty as well as a substitute for arial.sty.
        utilities.get_file('https://mirrors.ctan.org/fonts/urw/arial/latex/uarial.sty')
        utilities.link_or_copy('uarial.sty', 'arial.sty')

        # The file IJ4UQ_Bibliography_Style.bst has an issue causing 'r is an illegal
        # case-conversion string' error messages.
        # We apply a patch created via
        #   diff -u IJ4UQ_Bibliography_Style.bst IJ4UQ_Bibliography_Style-scoop.bst > IJ4UQ_Bibliography_Style.bst.patch
        utilities.apply_patch('IJ4UQ_Bibliography_Style.bst.patch')

        # Write the initialization time and version stamp.
        utilities.write_time_version_stamp()

    except:
        sys.exit(1)
